if not LibStub then return end

local icon = LibStub('LibDBIcon-1.0')

-- upvalues
local CalendarGetDate = CalendarGetDate
local CalendarGetEventInfo = CalendarGetEventInfo
local CalendarGetNumDayEvents = CalendarGetNumDayEvents
local OpenCalendar = OpenCalendar

local L = L

-- cache for events
local populated = false
local todaysEvents = {}
local todaysEventsCount = 0
local upcomingEvents = {}
local upcomingEventsCount = 0

obj = LibStub:GetLibrary('LibDataBroker-1.1'):NewDataObject('Broker_Calendar', {
	type = 'data source',
	text = 'Calendar',
	icon = 'Interface\\Icons\\INV_Misc_Rune_06',
})
local obj = obj
local frame = CreateFrame('frame')
frame:SetScript('OnEvent', function(self, event, ...) if self[event] then return self[event](self, event, ...) end end)
frame:RegisterEvent('PLAYER_ENTERING_WORLD')


local function ToggleMinimap()
	local hide = not CalendarDB.minimap.hide
	CalendarDB.minimap.hide = hide
	if hide then
		icon:Hide('Broker_Calendar')
	else
		icon:Show('Broker_Calendar')
	end
end

function frame:PLAYER_ENTERING_WORLD()
	-- PortalsDB.minimap is there for smooth upgrade of SVs from old version
	if not CalendarDB then
		CalendarDB = {}
		CalendarDB.minimap = {}
		CalendarDB.minimap.hide = false
		CalendarDB.version = 1
	end

	if icon then
		icon:Register('Broker_Calendar', obj, CalendarDB.minimap)
	end

	self:UnregisterEvent('PLAYER_ENTERING_WORLD')
  OpenCalendar()
end

local function printTwoCharNumber(number)
  if number < 10 then
    return '0'..number
  else
    return number
  end
end

local function printDays(num)
  if num == 1 then
    return ' ('..num..' day)'
  else
    return '('..num..' days)'
  end
end

local function populateEvents(day, month)
  -- today
  local numEvents = CalendarGetNumDayEvents(0, day)
  if numEvents > 0 then
    local title, hour, minute
    for eventIndex = 1, numEvents do
      title, hour, minute = CalendarGetDayEvent(0, day, eventIndex)
      todaysEvents[title] = {hour, minute}
      todaysEventsCount = todaysEventsCount + 1
    end
  end

  -- upcoming
  local numEvents, title, days
  local monthOffset = 0
  for dayOffset = 1, 7 do
    numEvents = CalendarGetNumDayEvents(0 + monthOffset, day + dayOffset)
    if numEvents ~= 0 then
      for eventIndex = 1, numEvents do
        title, _, _, _, _, _, _, _, _, _, _, _, _, days  = CalendarGetDayEvent(0 + monthOffset, day + dayOffset, eventIndex)
        if title ~= nil then
          if upcomingEvents[title] == nil and todaysEvents[title] == nil then
            upcomingEvents[title] = {day + dayOffset, month + monthOffset, days}
            upcomingEventsCount = upcomingEventsCount + 1
          end
        else
          monthOffset = monthOffset + 1
          day = 1 - dayOffset
        end
      end
    end
  end

  populated = true
end

local function showTodaysEvents()
  if todaysEventsCount == 0 then
    GameTooltip:AddLine('No events occuring today')
  else
    GameTooltip:AddDoubleLine('today:', 'starts', 0.9, 0.6, 0.2, 0.9, 0.6, 0.2)
    local title, hour, minute
    for title, time in pairs(todaysEvents) do
      GameTooltip:AddDoubleLine(title, printTwoCharNumber(time[1])..':'..printTwoCharNumber(time[2]), 1, 1, 0, 0.2, 1, 0.2)
    end
  end
end

local function showUpcomingEvents()
  if upcomingEventsCount > 0 then
    GameTooltip:AddLine(' ')
    GameTooltip:AddDoubleLine('comming soon:', 'starts (lasts)', 0.9, 0.6, 0.2, 0.9, 0.6, 0.2)
    for title, start in pairs(upcomingEvents) do
      GameTooltip:AddDoubleLine(title, printTwoCharNumber(start[1])..'.'..printTwoCharNumber(start[2])..'. '..printDays(start[3]), 1, 1, 0, 0.2, 1, 0.2)
    end
  end
end

-- All credit for this func goes to Tekkub and his picoGuild!
local function GetTipAnchor(frame)
		local x, y = frame:GetCenter()
		if not x or not y then return 'TOPLEFT', 'BOTTOMLEFT' end
		local hhalf = (x > UIParent:GetWidth() * 2 / 3) and 'RIGHT' or (x < UIParent:GetWidth() / 3) and 'LEFT' or ''
		local vhalf = (y > UIParent:GetHeight() / 2) and 'TOP' or 'BOTTOM'
		return vhalf..hhalf, frame, (vhalf == 'TOP' and 'BOTTOM' or 'TOP')..hhalf
end

function obj.OnClick(self, button)
  GameTooltip:Hide()
  GameTimeFrame_OnClick(GameTimeFrame)
end

function obj.OnLeave()
	GameTooltip:Hide() 
end

function obj.OnEnter(self)
 	GameTooltip:SetOwner(self, 'ANCHOR_NONE')
	GameTooltip:SetPoint(GetTipAnchor(self))
	GameTooltip:ClearLines()

  GameTooltip:AddLine('Broker Calendar', 1, 1, 0)
  local weekday, month, day, year = CalendarGetDate()
  GameTooltip:AddLine('Today is '..printTwoCharNumber(day)..'.'..printTwoCharNumber(month)..'.'..printTwoCharNumber(year), 1, 1, 0)
  GameTooltip:AddLine(' ')

  if not populated then
    populateEvents(day, month)
  end

  showTodaysEvents()
  showUpcomingEvents()

	GameTooltip:Show()
end

-- slashcommand definition
SlashCmdList['BROKER_CALENDAR'] = function() ToggleMinimap() end
SLASH_BROKER_CALENDAR1 =  '/b_calendar'
